@@ -54,6 +54,7 @@ class ForkCommand(osc.commandline.OscCommand):
 
     def run(self, args):
         from osc import conf as osc_conf
+        from osc import core as osc_core
         from osc import gitea_api
         from osc import obs_api
         from osc.git_scm import GitStore
@@ -83,27 +84,11 @@ class ForkCommand(osc.commandline.OscCommand):
             pkg = obs_api.Package.from_api(args.apiurl, project, package)
 
             if not args.no_devel_project:
-                # devel project is not set in package meta as usual but we parse it from "OBS:RejectBranch" attribute
-                try:
-                    attributes = obs_api.Attributes.from_api(args.apiurl, project, package, attr="OBS:RejectBranch").attribute_list
-                except HTTPError as e:
-                    if e.code != 404:
-                        raise
-                    attributes = []
-                except TypeError:
-                    # empty <attributes/> element, unable to instantiate Attributes model
-                    attributes = []
-                if attributes:
-                    attribute = attributes[0].value
-                    # the pattern starts with a non-greedy match so we capture the first url
-                    match = re.match(r".*?(https://[^ ]+).*", attribute)
-                    if match:
-                        devel_project_url = match.group(1)
-                        build_project = GitStore.get_build_project(devel_project_url)
-                        # override the package we're cloning with the one from the devel project
-                        use_devel_project = True
-                        project = build_project
-                        pkg = obs_api.Package.from_api(args.apiurl, project, package)
+                devel_project, devel_package = osc_core.show_devel_project(args.apiurl, project, package)
+                if devel_project:
+                    # override the package we're cloning with the one from the devel project
+                    pkg = obs_api.Package.from_api(args.apiurl, devel_project, devel_package)
+                    use_devel_project = True
 
             if not pkg.scmsync:
                 print(f"{tty.colorize('ERROR', 'red,bold')}: Forking is possible only with packages managed in Git (the <scmsync> element must be set in the package meta)")
@@ -119,9 +104,10 @@ class ForkCommand(osc.commandline.OscCommand):
 
         # parse gitea url, owner, repo and branch from the scmsync url
         if is_package:
-            parsed_scmsync_url = urllib.parse.urlparse(pkg.scmsync, scheme="https")
+            url_scmsync = pkg.scmsync
         else:
-            parsed_scmsync_url = urllib.parse.urlparse(project.scmsync, scheme="https")
+            url_scmsync = project.scmsync
+        parsed_scmsync_url = urllib.parse.urlparse(url_scmsync, scheme="https")
         url = urllib.parse.urlunparse((parsed_scmsync_url.scheme, parsed_scmsync_url.netloc, "", "", "", ""))
         owner, repo = parsed_scmsync_url.path.strip("/").split("/")
 
@@ -135,6 +121,9 @@ class ForkCommand(osc.commandline.OscCommand):
         # parse the right branch instead from .gitmodules
         #branch = parsed_scmsync_url.fragment or None
         branch = None
+        parsed_scmsync_url_query = urllib.parse.parse_qs(parsed_scmsync_url.query)
+        if "trackingbranch" in parsed_scmsync_url_query:
+            branch = parsed_scmsync_url_query["trackingbranch"][0]
 
         # find a credentials entry for url and OBS user (there can be multiple users configured for a single URL in the config file)
         gitea_conf = gitea_api.Config(args.gitea_config)
@@ -150,12 +139,10 @@ class ForkCommand(osc.commandline.OscCommand):
         print(f"Forking git repo {owner}/{repo} ...", file=sys.stderr)
 
         # the branch was not specified, fetch the default branch from the repo
-        if branch:
-            fork_branch = branch
-        else:
+        if not branch:
             repo_obj = gitea_api.Repo.get(gitea_conn, owner, repo)
             branch = repo_obj.default_branch
-            fork_branch = branch
+        fork_branch = branch
 
         # check if the scmsync branch exists in the source repo
         parent_branch_obj = gitea_api.Branch.get(gitea_conn, owner, repo, fork_branch)
@@ -172,7 +159,7 @@ class ForkCommand(osc.commandline.OscCommand):
 
         # XXX: implicit branch name should be forbidden; assumptions are bad
         fork_scmsync = urllib.parse.urlunparse(
-            (parsed_scmsync_url.scheme, parsed_scmsync_url.netloc, f"{fork_owner}/{fork_repo}", "", "", fork_branch)
+            (parsed_scmsync_url.scheme, parsed_scmsync_url.netloc, f"{fork_owner}/{fork_repo}", "", parsed_scmsync_url.query, fork_branch)
         )
 
         print()
