@@ -1,6 +1,7 @@
 import json
 import os
 import subprocess
+import sys
 import urllib.parse
 from pathlib import Path
 
@@ -98,9 +99,6 @@ class GitStore:
             return None
 
     def get_project_git_scm_store(self):
-        if not self.is_package:
-            return None
-
         path = self.abspath
         while path:
             if path == "/":
@@ -115,13 +113,47 @@ class GitStore:
         config_path = os.path.join(path, "_config")
         pbuild_path = os.path.join(path, "_pbuild")
         subdirs_path = os.path.join(path, "_subdirs")
+        manifest_path = os.path.join(path, "_manifest")
 
         # we always stop at the top-most directory that contains .git subdir
-        if not os.path.isfile(config_path) or os.path.isfile(pbuild_path):
+        if not (os.path.isfile(config_path) or os.path.isfile(pbuild_path)):
             # it's not a project, stop traversing and return
             return None
 
-        if os.path.isfile(subdirs_path):
+        if os.path.isfile(manifest_path):
+            if os.path.isfile(subdirs_path):
+                print("WARNING: Ignoring '_subdirs' file, using data from '_manifest'", file=sys.stderr)
+
+            # the _manifest file contains a list of project subdirs that contain packages and list of dirs which are packages
+            with open(manifest_path, "r") as f:
+                data = osc_yaml.yaml_load(f)
+
+            # ``packages`` is a list of directories which are packages
+            packages = data.get("packages", [])
+            packages_abspath = [os.path.abspath(os.path.join(path, i)) for i in packages]
+
+            # ``subdirectories`` is a list of directories, which have subdirectories which are packages
+            subdirs = data.get("subdirectories", [])
+            subdirs_abspath = [os.path.abspath(os.path.join(path, i)) for i in subdirs]
+
+            common_paths = set(packages) & set(subdirs)
+            if common_paths:
+                print(f"WARNING: _manifest contains conflicting entries between 'packages' and 'subdirectories': {sorted(common_paths)}", file=sys.stderr)
+
+            if self.abspath in packages_abspath:
+                # we're in a path defined in 'packages' -> it's a package
+                pass
+            elif self.abspath in subdirs_abspath:
+                # paths listed in ``subdirectories`` are never packages, their subdirs are
+                return None
+            elif os.path.abspath(os.path.join(self.abspath, "..")) not in subdirs_abspath:
+                # we're outside paths specified in 'subdirectories' -> not a package
+                return None
+            else:
+                # we're in a subdir of a directory listed in 'subdirectories' -> it's a package
+                pass
+
+        elif os.path.isfile(subdirs_path):
             # the _subdirs file contains a list of project subdirs that contain packages
             with open(subdirs_path, "r") as f:
                 data = osc_yaml.yaml_load(f)
@@ -210,13 +242,26 @@ class GitStore:
 
     @property
     def apiurl(self):
+        from ..obs_scm import Store
+
         if not self._apiurl:
-            if self.is_package and self.project_store:
+            if not self._apiurl and self.project_store:
+                # get apiurl from .osc that is at the same level as project .git
+                try:
+                    store = Store(self.project_store.abspath)
+                    store.assert_is_project()
+                    self._apiurl = store.apiurl
+                except oscerr.NoWorkingCopy:
+                    pass
+
+            if not self._apiurl and self.is_package and self.project_store:
                 # read apiurl from parent directory that contains a project with .osc metadata
                 self._apiurl = self.project_store.apiurl
+
             if not self._apiurl:
                 # HACK: use the currently configured apiurl
                 self._apiurl = osc_conf.config["apiurl"]
+
         return self._apiurl
 
     @apiurl.setter
@@ -225,17 +270,28 @@ class GitStore:
 
     @property
     def project(self):
+        from ..obs_scm import Store
+
         if not self._project:
             if self.is_package:
                 # handle _project in a package
 
-                if self.project_store:
+                if not self._project and self.project_store:
+                    # get project from .osc that is at the same level as project .git
+                    try:
+                        store = Store(self.project_store.abspath)
+                        store.assert_is_project()
+                        self._project = store.project
+                    except oscerr.NoWorkingCopy:
+                        pass
+
+                if not self._project and self.project_store:
                     # read project from detected project store
                     self._project = self.project_store.project
 
                 if not self._project:
                     # read project from Gitea (identical owner, repo: _ObsPrj, file: project.build)
-                    origin = self._run_git(["remote", "get-url", "origin"])
+                    origin = self._run_git(["remote", "get-url", self.current_remote])
                     self._project = self.get_build_project(origin)
 
             else:
@@ -261,7 +317,7 @@ class GitStore:
     @property
     def package(self):
         if self._package is None:
-            origin = self._run_git(["remote", "get-url", "origin"])
+            origin = self._run_git(["remote", "get-url", self.current_remote])
             self._package = Path(urllib.parse.urlsplit(origin).path).stem
         return self._package
 
@@ -305,6 +361,22 @@ class GitStore:
         value = json.dumps(value)
         self._set_option(name, value)
 
+    def _get_list_option(self, name):
+        result = self._get_option(name)
+        if result is None:
+            return None
+        result = json.loads(result)
+        self._check_type(name, result, list)
+        return result
+
+    def _set_list_option(self, name, value):
+        if value is None:
+            self._unset_option(name)
+            return
+        self._check_type(name, value, list)
+        value = json.dumps(value)
+        self._set_option(name, value)
+
     @property
     def last_buildroot(self):
         self.assert_is_package()
@@ -325,9 +397,46 @@ class GitStore:
         }
         self._set_dict_option("last-buildroot", value)
 
+    @property
+    def build_repositories(self):
+        from ..core import Repo
+
+        self.assert_is_package()
+        repos = self._get_list_option("build-repositories")
+        if repos is None:
+            return None
+        return [Repo(**i) for i in repos]
+
+    @build_repositories.setter
+    def build_repositories(self, value):
+        from ..core import Repo
+
+        self.assert_is_package()
+        repos = []
+        if value is not None:
+            for i in value:
+                if not isinstance(i, Repo):
+                    raise ValueError(f"The value is not an instance of 'Repo': {i}")
+                repos.append(i.dict())
+        self._set_list_option("build-repositories", repos)
+
     @property
     def scmurl(self):
         try:
-            return self._run_git(["remote", "get-url", "origin"])
+            return self._run_git(["remote", "get-url", self.current_remote])
         except subprocess.CalledProcessError:
             return None
+
+    @property
+    def current_remote(self):
+        result = None
+        try:
+            result = self._run_git(["rev-parse", "--abbrev-ref", "@{u}"])
+            if result:
+                result = result.split("/")[0]
+        except subprocess.CalledProcessError:
+            pass
+
+        if result:
+            return result
+        return "origin"
