@@ -5,19 +5,22 @@ from typing import Dict
 from typing import List
 from typing import Optional
 
-import ruamel.yaml
-
 from osc import oscerr
+from osc.util import yaml as osc_yaml
 from osc.util.models import BaseModel
 from osc.util.models import Field
 
 
 class Login(BaseModel):
+    # Any time the fields change, make corresponding changes in the ENV variables in the following places:
+    # * Config.get_login() in this file - update ENV variables passed to Login()
+    # * GitObsMainCommand.init_arguments in ../commandline_git.py - update help
     name: str = Field()  # type: ignore[assignment]
     url: str = Field()  # type: ignore[assignment]
     user: str = Field()  # type: ignore[assignment]
     token: str = Field()  # type: ignore[assignment]
     ssh_key: Optional[str] = Field()  # type: ignore[assignment]
+    git_uses_http: Optional[bool] = Field()  # type: ignore[assignment]
     default: Optional[bool] = Field()  # type: ignore[assignment]
 
     class AlreadyExists(oscerr.OscBaseError):
@@ -35,9 +38,16 @@ class Login(BaseModel):
 
         def __str__(self):
             if self.kwargs == {"name": None}:
-                return "Could not find a default Gitea config entry"
+                return (
+                    "Could not find a default Gitea config entry\n"
+                    " * either set the default entry by running `git-obs login update <login> --set-as-default`\n"
+                    " * or run the command with `-G/--gitea-login <login>` option"
+                )
             kwargs_str = ", ".join([f"{key}={value}" for key, value in self.kwargs.items()])
-            return f"Could not find a matching Gitea config entry: {kwargs_str}"
+            return (
+                f"Could not find a matching Gitea config entry: {kwargs_str}\n"
+                " * see `git-obs login list` for valid entries"
+            )
 
     def __init__(self, **kwargs):
         # ignore extra fields
@@ -49,14 +59,16 @@ class Login(BaseModel):
     def to_human_readable_string(self, *, show_token: bool = False):
         from osc.output import KeyValueTable
 
-        table = KeyValueTable()
+        table = KeyValueTable(min_key_length=20)
         table.add("Name", self.name, color="bold")
         if self.default:
             table.add("Default", "true", color="bold")
         table.add("URL", self.url)
         table.add("User", self.user)
         if self.ssh_key:
-            table.add("SSH Key", self.ssh_key)
+            table.add("Private SSH key path", self.ssh_key)
+        if self.git_uses_http:
+            table.add("Git uses http(s)", "yes" if self.git_uses_http else "no")
         if show_token:
             # tokens are stored in the plain text, there's not reason to protect them too much
             # let's only hide them from the output by default
@@ -80,27 +92,29 @@ class Config:
     def _read(self) -> Dict[str, Any]:
         try:
             with open(self.path, "r") as f:
-                yaml = ruamel.yaml.YAML()
-                return yaml.load(f)
+                return osc_yaml.yaml_load(f)
         except FileNotFoundError:
             return {}
 
     def _write(self, data):
-        yaml = ruamel.yaml.YAML()
-        yaml.default_flow_style = False
-        buf = io.StringIO()
-        yaml.dump(data, buf)
-        buf.seek(0)
-        text = buf.read()
-
         os.makedirs(os.path.dirname(self.path), mode=0o700, exist_ok=True)
         with open(self.path, "w") as f:
-            f.write(text)
+            osc_yaml.yaml_dump(data, f)
 
     def list_logins(self) -> List[Login]:
         data = self._read()
         result = []
         for i in data.get("logins", []):
+            login_name = i["name"]
+
+            # override values from GIT_OBS_LOGIN_{login_name}_{field_name.upper()}
+            for field_name in Login.__fields__.keys():
+                if field_name in ("name", "url"):
+                    continue
+                value = os.environ.get(f"GIT_OBS_LOGIN_{login_name}_{field_name.upper()}", None)
+                if value:
+                    i[field_name] = value
+
             login = Login(**i)
             result.append(login)
         return result
@@ -110,6 +124,22 @@ class Config:
         Return ``Login`` object for the given ``name``.
         If ``name`` equals to ``None``, return the default ``Login``.
         """
+
+        if name is None:
+            # if login name was not explicitly specified, check if there are credentials specified in env
+            env_gitea_url = os.environ.get("GIT_OBS_GITEA_URL", None)
+            env_gitea_user = os.environ.get("GIT_OBS_GITEA_USER", None)
+            env_gitea_token = os.environ.get("GIT_OBS_GITEA_TOKEN", None)
+            env_ssh_key = os.environ.get("GIT_OBS_GITEA_SSH_KEY", None)
+            if env_gitea_url and env_gitea_user and env_gitea_token:
+                return Login(
+                    name="<ENV>",
+                    url=env_gitea_url,
+                    user=env_gitea_user,
+                    token=env_gitea_token,
+                    ssh_key=env_ssh_key,
+                )
+
         for login in self.list_logins():
             if name is None and login.default:
                 return login
@@ -181,6 +211,7 @@ class Config:
         new_user: Optional[str] = None,
         new_token: Optional[str] = None,
         new_ssh_key: Optional[str] = None,
+        new_git_uses_http: Optional[bool] = None,
         set_as_default: Optional[bool] = None,
     ) -> Login:
         login = self.get_login(name)
@@ -195,6 +226,16 @@ class Config:
             login.token = new_token
         if new_ssh_key is not None:
             login.ssh_key = new_ssh_key
+
+        if new_git_uses_http is None:
+            # keep the original value
+            pass
+        elif new_git_uses_http:
+            login.git_uses_http = True
+        else:
+            # remove from the config instead of setting to False
+            login.git_uses_http = None
+
         if set_as_default:
             login.default = True
 
@@ -205,6 +246,11 @@ class Config:
         for entry in data["logins"]:
             if entry.get("name", None) == name:
                 entry.update(login.dict())
+
+                # remove keys with no value
+                for key, value in entry.copy().items():
+                    if value is None:
+                        del entry[key]
             else:
                 if set_as_default:
                     entry.pop("default", None)
