@@ -4,17 +4,14 @@ from typing import List
 from typing import Optional
 from typing import Tuple
 
+from .common import GiteaModel
 from .connection import Connection
 from .connection import GiteaHTTPResponse
 from .user import User
 
 
 @functools.total_ordering
-class PullRequest:
-    def __init__(self, data, *, response: Optional[GiteaHTTPResponse] = None):
-        self._data = data
-        self._response = response
-
+class PullRequest(GiteaModel):
     def __eq__(self, other):
         (self.base_owner, self.base_repo, self.number) == (other.base_owner, other.base_repo, other.number)
 
@@ -24,13 +21,27 @@ class PullRequest:
     @classmethod
     def split_id(cls, pr_id: str) -> Tuple[str, str, int]:
         """
-        Split <owner>/<repo>#<number> into individual components and return them in a tuple.
+        Split <owner>/<repo>#<number> or <owner>/<repo>!<number> into individual components and return them in a tuple.
         """
-        match = re.match(r"^([^/]+)/([^/]+)#([0-9]+)$", pr_id)
+        match = re.match(r"^([^/]+)/([^/]+)[#!]([0-9]+)$", pr_id)
         if not match:
             raise ValueError(f"Invalid pull request id: {pr_id}")
         return match.group(1), match.group(2), int(match.group(3))
 
+    @staticmethod
+    def get_owner_repo_number(url: str) -> Tuple[str, str, int]:
+        """
+        Parse pull request URL such as http(s)://example.com:<port>/<owner>/<repo>/pulls/<number>
+        and return (owner, repo, number) tuple.
+        """
+        import urllib.parse
+
+        parsed_url = urllib.parse.urlparse(url)
+        path = parsed_url.path
+        owner, repo, pulls, number = path.strip("/").split("/")
+        assert pulls in ("pulls", "issues")
+        return owner, repo, int(number)
+
     @property
     def is_pull_request(self):
         # determine if we're working with a proper pull request or an issue without pull request details
@@ -116,12 +127,16 @@ class PullRequest:
     def head_owner(self) -> Optional[str]:
         if not self.is_pull_request:
             return None
+        if self._data["head"]["repo"] is None:
+            return None
         return self._data["head"]["repo"]["owner"]["login"]
 
     @property
     def head_repo(self) -> Optional[str]:
         if not self.is_pull_request:
             return None
+        if self._data["head"]["repo"] is None:
+            return None
         return self._data["head"]["repo"]["name"]
 
     @property
@@ -140,8 +155,16 @@ class PullRequest:
     def head_ssh_url(self) -> Optional[str]:
         if not self.is_pull_request:
             return None
+        if self._data["head"]["repo"] is None:
+            return None
         return self._data["head"]["repo"]["ssh_url"]
 
+    @property
+    def merge_commit(self) -> Optional[str]:
+        if not self.is_pull_request:
+            return None
+        return self._data["merge_commit_sha"]
+
     @property
     def url(self) -> str:
         # HACK: search API returns issues, the URL needs to be transformed to a pull request URL
@@ -359,12 +382,12 @@ class PullRequest:
             "created": created,
             "mentioned": mentioned,
             "review_requested": review_requested,
-            # HACK: limit=-1 doesn't work, the request gets stuck; we need to use a high number to avoid pagination
-            "limit": 10**6,
+            "limit": 50,
         }
         url = conn.makeurl("repos", "issues", "search", query=q)
-        response = conn.request("GET", url)
-        obj_list = [cls(i, response=response) for i in response.json()]
+        obj_list = []
+        for response in conn.request_all_pages("GET", url):
+            obj_list.extend([cls(i, response=response) for i in response.json()])
         return obj_list
 
     @classmethod
@@ -383,7 +406,11 @@ class PullRequest:
         :param repo: Name of the repo.
         :param number: Number of the pull request in the repo.
         """
-        url = conn.makeurl("repos", owner, repo, "pulls", f"{number}.patch")
+        q = {
+            "binary": 0,
+        }
+        # XXX: .patch suffix doesn't work with binary=0
+        url = conn.makeurl("repos", owner, repo, "pulls", f"{number}.diff", query=q)
         response = conn.request("GET", url)
         return response.data
 
@@ -426,6 +453,7 @@ class PullRequest:
         *,
         msg: Optional[str] = None,
         commit: Optional[str] = None,
+        reviewer: Optional[str] = None,
     ) -> GiteaHTTPResponse:
         """
         Approve review in a pull request.
@@ -435,6 +463,12 @@ class PullRequest:
             if pr_obj.head_commit != commit:
                 raise RuntimeError("The pull request '{owner}/{repo}#{number}' has changed during the review")
 
+        if reviewer:
+            # group review bot is controlled via messages in comments
+            msg = f"@{reviewer} : approve\n\n" + (msg or "")
+            msg = msg.strip()
+            return cls.add_comment(conn, owner, repo, number, msg=msg)
+
         url = conn.makeurl("repos", owner, repo, "pulls", str(number), "reviews")
         # XXX[dmach]: commit_id has no effect; I thought it's going to approve if the commit matches with head and errors out otherwise
         json_data = {
@@ -454,10 +488,17 @@ class PullRequest:
         *,
         msg: str,
         commit: Optional[str] = None,
+        reviewer: Optional[str] = None,
     ) -> GiteaHTTPResponse:
         """
         Decline review (request changes) in a pull request.
         """
+        if reviewer:
+            # group review bot is controlled via messages in comments
+            msg = f"@{reviewer} : decline\n\n" + (msg or "")
+            msg = msg.strip()
+            return cls.add_comment(conn, owner, repo, number, msg=msg)
+
         url = conn.makeurl("repos", owner, repo, "pulls", str(number), "reviews")
         json_data = {
             "event": "REQUEST_CHANGES",
@@ -492,3 +533,63 @@ class PullRequest:
             conn.request("POST", url, json_data=json_data, context={"owner": owner, "repo": repo})
         except AutoMergeAlreadyScheduled:
             pass
+
+    @classmethod
+    def cancel_scheduled_merge(
+        cls,
+        conn: Connection,
+        owner: str,
+        repo: str,
+        number: int,
+    ) -> GiteaHTTPResponse:
+        """
+        Cancel scheduled merge of a pull request.
+        """
+        from .exceptions import GiteaException
+
+        url = conn.makeurl("repos", owner, repo, "pulls", str(number), "merge")
+        try:
+            conn.request("DELETE", url, context={"owner": owner, "repo": repo})
+        except GiteaException as e:
+            # Gitea returns 404 when there's no scheduled merge or if the pull request doesn't exist
+            # the error message is the same and it's not possible to distinguish between the two cases.
+            if e.status != 404:
+                raise
+
+    @classmethod
+    def close(
+        cls,
+        conn: Connection,
+        owner: str,
+        repo: str,
+        number: int,
+    ) -> "PullRequest":
+        """
+        Close a pull request.
+        """
+        url = conn.makeurl("repos", owner, repo, "pulls", str(number))
+        json_data = {
+            "state": "closed",
+        }
+        response = conn.request("PATCH", url, json_data=json_data, context={"owner": owner, "repo": repo})
+        obj = cls(response.json(), response=response, conn=conn)
+        return obj
+
+    @classmethod
+    def reopen(
+        cls,
+        conn: Connection,
+        owner: str,
+        repo: str,
+        number: int,
+    ) -> "PullRequest":
+        """
+        Reopen a pull request.
+        """
+        url = conn.makeurl("repos", owner, repo, "pulls", str(number))
+        json_data = {
+            "state": "open",
+        }
+        response = conn.request("PATCH", url, json_data=json_data, context={"owner": owner, "repo": repo})
+        obj = cls(response.json(), response=response, conn=conn)
+        return obj
